﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using Newtonsoft.Json;

namespace dotNetPaymentInterfaceDemo
{
    class Program
    {
        //商户号
        private const string merchantno = "xxxxxxxxxxxxxxxxxxxxxx";
        //终端号
        private const string terminal = "xxxxxxxx";
        //秘钥
        private const string token = "xxxxxxxxxxxxxxxxxxxxxxxxxxxx";
        //支付地址
        private const string url = "http://test.lcsw.cn:8045/lcsw";
        static void Main()
        {
            //刷卡支付
            barcodepay("xxxxxxxxxxxxxxxxxx");
            //扫码支付
            prepay();
            // 公众号预支付（统一下单）
            jspay();
            //小程序支付
            minipay();

            //查询
            query("xxxxxxxxxxxxxxxxxxxxxxxxxx");
            query("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx", "xxxxxxxxxxxxx");

            refund("1", "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx");

            //聚合码
            qrpay();
        }


        /// <summary>
        /// 付款码支付
        /// </summary>
        /// <param name="auth_no">授权码，客户的付款码</param>
        private static void barcodepay(string auth_no)
        {
            var dic = new Dictionary<string, string>
            {
                {"pay_ver", "110"},
                {"pay_type", "000"},
                {"service_id", "010"},
                {"merchant_no", merchantno},
                {"terminal_id", terminal},
                { "terminal_trace", Guid.NewGuid().ToString("N")},
                {"terminal_time", DateTime.Now.ToString("yyyyMMddHHmmss")},
                {"auth_no", auth_no},
                {"total_fee", "1"} //金额 单位分
            };
            dic.Add("key_sign", Md5Encryption(dic));

            var result = HttpPost(url + "/pay/110/barcodepay", dic);
        }

        /// <summary>
        /// 扫码支付 因微信政策调整【微信间联关闭APP支付和Native支付通知】 本接口微信支付将于2019年9月15号后关闭，建议存量商户调整为聚合码支付接口或JSAPI公众号支付。
        /// </summary> 
        private static void prepay()
        {
            var dic = new Dictionary<string, string>
            {
                {"pay_ver", "110"},
                {"pay_type", "010"},
                {"service_id", "011"},
                {"merchant_no", merchantno},
                {"terminal_id", terminal},
                { "terminal_trace", Guid.NewGuid().ToString("N")},
                {"terminal_time", DateTime.Now.ToString("yyyyMMddHHmmss")},
                {"total_fee", "1"} //金额 单位分
            };
            dic.Add("key_sign", Md5Encryption(dic));

            var result = HttpPost(url + "/pay/110/prepay", dic);
        }

        /// <summary>
        /// 公众号预支付
        /// </summary>
        private static void jspay()
        {
            var dic = new Dictionary<string, string>
            {
                {"pay_ver", "110"},
                {"pay_type", "010"},
                {"service_id", "012"},
                {"merchant_no", merchantno},
                {"terminal_id", terminal},
                { "terminal_trace", Guid.NewGuid().ToString("N")},
                {"terminal_time", DateTime.Now.ToString("yyyyMMddHHmmss")},
                {"total_fee", "1"}, //金额 单位分
                {"open_id","xxxxxxxxxxxxxxxxx" } //用户标识（微信openid，支付宝userid），pay_type为010及020时需要传入
            };
            dic.Add("key_sign", Md5Encryption(dic));

            var result = HttpPost(url + "/pay/110/jspay", dic);
        }

        /// <summary>
        /// 小程序 通过我们接口的返参调用微信官方的支付，我们接口如下，微信官方请查看微信官方文档
        /// </summary>
        private static void minipay()
        {
            var dic = new Dictionary<string, string>
            {
                {"pay_ver", "110"},
                {"pay_type", "010"},
                {"service_id", "015"},
                {"merchant_no", merchantno},
                {"terminal_id", terminal},
                { "terminal_trace", Guid.NewGuid().ToString("N")},
                {"terminal_time", DateTime.Now.ToString("yyyyMMddHHmmss")},
                {"total_fee", "1"}, //金额 单位分
                {"open_id","xxxxxxxxxxxxxxxxxxxxxx" } //用户标识（微信openid，支付宝userid），pay_type为010及020时需要传入
            };
            dic.Add("key_sign", Md5Encryption(dic));

            var result = HttpPost(url + "/pay/110/minipay", dic);
        }

        /// <summary>
        /// 聚合码支付
        /// </summary>
        private static void qrpay()
        {
            var dic = new Dictionary<string, string>
            {
                {"pay_ver", "110"},
                {"pay_type", "000"},
                {"service_id", "016"},
                {"merchant_no", merchantno},
                {"terminal_id", terminal},
                { "terminal_trace", Guid.NewGuid().ToString("N")},
                {"terminal_time", DateTime.Now.ToString("yyyyMMddHHmmss")},
                {"total_fee", "1"} //金额 单位分
            };
            dic.Add("key_sign", Md5Encryption(dic));

            var result = HttpPost(url + "/pay/110/qrpay", dic);
        }

        #region 查询 查询提供两种方式 1 用订单号查询 2使用 流水+时间查询
        //方式1 订单号查询
        private static void query(string out_trade_no)
        {
            var dic = new Dictionary<string, string>
            {
                {"pay_ver", "110"},
                {"pay_type", "000"},
                {"service_id", "020"},
                {"merchant_no", merchantno},
                {"terminal_id", terminal},
                { "terminal_trace", Guid.NewGuid().ToString("N")},
                {"terminal_time", DateTime.Now.ToString("yyyyMMddHHmmss")},
                {"pay_trace", ""},
                {"pay_time", ""},
                {"out_trade_no", out_trade_no},

            };
            dic.Add("key_sign", Md5Encryption(dic));

            var result = HttpPost(url + "/pay/110/query", dic);
        }

        //方式2 流水+时间
        private static void query(string pay_trace,string pay_time)
        {
            var dic = new Dictionary<string, string>
            {
                {"pay_ver", "110"},
                {"pay_type", "000"},
                {"service_id", "020"},
                {"merchant_no", merchantno},
                {"terminal_id", terminal},
                { "terminal_trace", Guid.NewGuid().ToString("N")},
                {"terminal_time", DateTime.Now.ToString("yyyyMMddHHmmss")},
                {"pay_trace", pay_trace},
                {"pay_time", pay_time},
                {"out_trade_no", ""}

            };
            dic.Add("key_sign", Md5Encryption(dic));

            var result = HttpPost(url + "/pay/110/query", dic);
        }
        #endregion
  
        private static void refund(string fee,string out_trade_no)
        {
            var dic = new Dictionary<string, string>
            {
                {"pay_ver", "110"},
                {"pay_type", "000"},
                {"service_id", "030"},
                {"merchant_no", merchantno},
                {"terminal_id", terminal},
                { "terminal_trace", Guid.NewGuid().ToString("N")},
                {"terminal_time", DateTime.Now.ToString("yyyyMMddHHmmss")},
                {"refund_fee", fee},
                {"out_trade_no",out_trade_no }

            };
            dic.Add("key_sign", Md5Encryption(dic));

            var result = HttpPost(url + "/pay/110/refund", dic);
        }

        /// <summary>
        /// 计算签名
        /// </summary>
        /// <param name="dic"></param>
        /// <returns></returns>
        public static string Md5Encryption(Dictionary<string, string> dic)
        {
            StringBuilder sb = new StringBuilder();
            dic = dic.OrderBy(a => a.Key).ToDictionary(o => o.Key, p => p.Value);
            foreach (var item in dic)
            {
                if (item.Value != null)
                {
                    sb.Append(item.Key + "=" + item.Value + "&");
                }
            }
            sb.Append("access_token=" + token);
            var md5 = System.Security.Cryptography.MD5.Create();
            var bs = md5.ComputeHash(Encoding.UTF8.GetBytes(sb.ToString()));
            var str = new StringBuilder();
            foreach (var b in bs)
            {
                str.Append(b.ToString("x2"));
            }
            return str.ToString().ToUpper();
        }

        #region http Post 
        private static string HttpPost(string url, Dictionary<string, string> dic)
        {
            try
            {
                string result = string.Empty;
                HttpWebRequest req = (HttpWebRequest)WebRequest.Create(url);
                req.Method = "POST";
                req.Timeout = 8000;
                req.ContentType = "application/json";
                byte[] data = Encoding.UTF8.GetBytes(JsonConvert.SerializeObject(dic));
                req.ContentLength = data.Length;
                using (Stream reqStream = req.GetRequestStream())
                {
                    reqStream.Write(data, 0, data.Length);
                    reqStream.Close();
                }
                HttpWebResponse resp = (HttpWebResponse)req.GetResponse();
                Stream stream = resp.GetResponseStream();
                using (StreamReader reader = new StreamReader(stream, Encoding.UTF8))
                {
                    result = reader.ReadToEnd();
                }
                stream.Close();
                return result;
            }
            catch (WebException ex)
            {
                return null;
            }
        }
        #endregion
    }
}
